""" Add unit and functional testing support to setuptools-driven eggs.
"""
from __future__ import print_function

from setuptools.command.test import ScanningLoader
from setuptools.command.test import test as BaseCommand


def skipLayers(suite, _result=None):
    """ Walk the suite returned by setuptools' testloader.

    o Skip any tests which have a 'layer' defined.
    """
    from unittest import TestSuite
    if _result is None:
        _result = TestSuite()
    # Sometimes test suites do not have tests, like DocFileSuite.
    if not suite._tests:
        _result.addTest(suite)
        return _result
    for test in suite._tests:
        layer = getattr(test, 'layer', None)
        if layer is not None:
            continue
        if isinstance(test, TestSuite):
            skipLayers(test, _result)
        else:
            _result.addTest(test)
    return _result


class SkipLayers(ScanningLoader):
    """
    Load only unit tests (those which have no layer associated with
    them).

    * Running the tests using 'setup.py test' cannot, by default,
      drive the full testrunner, with its support for layers (in
      functional tests).  This loader allows the command to work, by
      running only those tests which don't need the layer support.

    * To run layer-dependent tests, use 'setup.py ftest' (see below
      for adding the command to your setup.py).

    * To use this loader your package add the following your 'setup()'
      call::

        setup(
            ...
            setup_requires=[
                'eggtestinfo' # captures testing metadata in EGG-INFO
            ],
            tests_require=['zope.testrunner', ],
            ...
            test_loader='zope.testrunner.eggsupport:SkipLayers',
            ...
        )
    """
    def loadTestsFromModule(self, module):
        return skipLayers(
            ScanningLoader.loadTestsFromModule(self, module))

    def loadTestsFromNames(self, testNames, module):
        return skipLayers(
            ScanningLoader.loadTestsFromNames(self, testNames, module))


def print_usage():
    print('python setup.py ftest')
    print()
    print(ftest.__doc__)


class ftest(BaseCommand):
    """
    Run unit and functional tests after an in-place build.

    * Note that this command runs *all* tests (unit *and* functional).

    * This command does not provide any of the configuration options which
      the usual testrunner provided by 'zope.testrunner' offers:  it is
      intended to allow easy verification that a package has been installed
      correctly via setuptools, but is not likely to be useful for
      developers working on the package.

    * Developers working on the package will likely prefer to work with
      the stock testrunner, e.g., by using buildout with a recipe which
      configures the testrunner as a standalone script.

    * To use this in your package add the following to the 'entry_points'
      section::

          setup(
              ...
              setup_requires=[
                  'zope.testrunner',
                  'eggtestinfo' # captures testing metadata in EGG-INFO
              ],
              ...
          )
    """
    description = "Run all functional and unit tests after in-place build"
    user_options = []
    help_options = [('usage', '?', 'Show usage', print_usage)]

    def initialize_options(self):
        pass  # suppress normal handling

    def finalize_options(self):
        pass  # suppress normal handling

    def run(self):
        from zope.testrunner import run
        args = ['IGNORE_ME']

        dist = self.distribution
        for src_loc in (dist.package_dir.values() or ['.']):
            args += ['--test-path', src_loc]

        if dist.install_requires:
            dist.fetch_build_eggs(dist.install_requires)

        if dist.tests_require:
            dist.fetch_build_eggs(dist.tests_require)

        def _run():
            run(args=args)

        self.with_project_on_sys_path(_run)



##############################################################################
#
# Copyright (c) 2004-2008 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Layer definitions
"""
import unittest


class UnitTests(object):
    """A layer for gathering all unit tests."""


class EmptyLayer(object):
    """An empty layer to start spreading out subprocesses."""

    __bases__ = ()
    __module__ = ''


def EmptySuite():
    suite = unittest.TestSuite()
    suite.layer = EmptyLayer()
    return suite



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Set up testing environment
"""

import sys
import traceback

import zope.exceptions.exceptionformatter

import zope.testrunner.feature


try:
    _iter_chain = traceback._iter_chain
except AttributeError:
    # Python 3.5
    def _iter_chain(exc, custom_tb=None, seen=None):
        if seen is None:
            seen = set()
        seen.add(exc)
        its = []
        context = exc.__context__
        cause = exc.__cause__
        if cause is not None and cause not in seen:
            its.append(_iter_chain(cause, False, seen))
            its.append([(traceback._cause_message, None)])
        elif (context is not None and
              not exc.__suppress_context__ and
              context not in seen):
            its.append(_iter_chain(context, None, seen))
            its.append([(traceback._context_message, None)])
        its.append([(exc, custom_tb or exc.__traceback__)])
        # itertools.chain is in an extension module and may be unavailable
        for it in its:
            for x in it:
                yield x


def format_exception(t, v, tb, limit=None, chain=None):
    if chain:
        values = _iter_chain(v, tb)
    else:
        values = [(v, tb)]
    fmt = zope.exceptions.exceptionformatter.TextExceptionFormatter(
        limit=None, with_filenames=True)
    for v, tb in values:
        if isinstance(v, str):
            return v
        return fmt.formatException(t, v, tb)


def print_exception(t, v, tb, limit=None, file=None, chain=None):
    if chain:
        values = _iter_chain(v, tb)
    else:
        values = [(v, tb)]
    if file is None:
        file = sys.stdout
    for v, tb in values:
        file.writelines(format_exception(t, v, tb, limit))


class Traceback(zope.testrunner.feature.Feature):

    active = True

    def global_setup(self):
        self.old_format = traceback.format_exception
        traceback.format_exception = format_exception

        self.old_print = traceback.print_exception
        traceback.print_exception = print_exception

    def global_teardown(self):
        traceback.format_exception = self.old_format
        traceback.print_exception = self.old_print



##############################################################################
#
# Copyright (c) 2010 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Unit tests for the testrunner's subunit integration."""


import io
import sys
import unittest

from zope.testrunner import formatter


try:
    unichr
except NameError:
    unichr = chr  # Python 3


try:
    import subunit
    subunit
except ImportError:
    def test_suite():
        return unittest.TestSuite()
else:

    class TestSubunitTracebackPrintingMixin(object):

        def makeByteStringFailure(self, text, encoding):
            try:
                if sys.version_info[0] < 3:
                    # On Python 2, note that this deliberately throws a
                    # string of bytes instead of a unicode object.  This
                    # simulates errors thrown by utf8-encoded doctests.
                    bytestr = text.encode(encoding)
                    self.fail(bytestr)
                else:
                    # On Python 3, it's more accurate to just use the
                    # Unicode text directly.
                    self.fail(text)
            except self.failureException:
                return sys.exc_info()

        def test_print_failure_containing_utf8_bytestrings(self):
            exc_info = self.makeByteStringFailure(unichr(6514), 'utf8')
            self.subunit_formatter.test_failure(self, 0, exc_info)
            assert b"AssertionError: \xe1\xa5\xb2" in self.output.getvalue()
            # '\xe1\xa5\xb2'.decode('utf-8') == unichr(6514)

        def test_print_error_containing_utf8_bytestrings(self):
            exc_info = self.makeByteStringFailure(unichr(6514), 'utf8')
            self.subunit_formatter.test_error(self, 0, exc_info)
            assert b"AssertionError: \xe1\xa5\xb2" in self.output.getvalue()
            # '\xe1\xa5\xb2'.decode('utf-8') == unichr(6514)

        @unittest.skipIf(
            sys.version_info[0] >= 3,
            'Tracebacks are always Unicode on Python 3')
        def test_print_failure_containing_latin1_bytestrings(self):
            exc_info = self.makeByteStringFailure(unichr(241), 'latin1')
            self.subunit_formatter.test_failure(self, 0, exc_info)
            assert b"AssertionError: \xef\xbf\xbd" in self.output.getvalue()
            # '\xef\xbf\xbd'.decode('utf-8') = unichr(0xFFFD)

    class TestSubunitTracebackPrinting(
            TestSubunitTracebackPrintingMixin, unittest.TestCase):

        def setUp(self):
            class FormatterOptions:
                verbose = False
            options = FormatterOptions()

            self.output = io.BytesIO()
            self.subunit_formatter = formatter.SubunitOutputFormatter(
                options, stream=self.output)

    class TestSubunitV2TracebackPrinting(
            TestSubunitTracebackPrintingMixin, unittest.TestCase):

        def setUp(self):
            class FormatterOptions:
                verbose = False
            options = FormatterOptions()

            self.output = io.BytesIO()
            self.subunit_formatter = formatter.SubunitV2OutputFormatter(
                options, stream=self.output)

    def test_suite():
        return unittest.defaultTestLoader.loadTestsFromName(__name__)



##############################################################################
#
# Copyright (c) 2004, 2013 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
# This package is developed by the Zope Toolkit project, documented here:
# http://docs.zope.org/zopetoolkit
# When developing and releasing this package, please follow the documented
# Zope Toolkit policies as described by this documentation.
##############################################################################
import os
import sys

from setuptools import setup
from setuptools.command.test import test


version = '5.6'

INSTALL_REQUIRES = [
    'setuptools',
    'six',
    'zope.exceptions',
    'zope.interface',
]

TESTS_REQUIRE = [
    'zope.testing',
]

EXTRAS_REQUIRE = {
    'test': TESTS_REQUIRE,
    'subunit': [
        'python-subunit >= 0.0.11',
        'testtools >= 0.9.30',
    ],
    'docs': [
        'Sphinx',
        'sphinxcontrib-programoutput',
    ],
}


CUSTOM_TEST_TEMPLATE = """\
import sys
sys.path = %r

try:
    import coverage
except ImportError:
    pass
else:
    coverage.process_startup()

import os
os.chdir(%r)

# The following unused imports are dark magic that makes the tests pass on
# Python 3.5 on Travis CI.  I do not understand why.
import zope.exceptions.exceptionformatter
import zope.testing

import zope.testrunner
if __name__ == '__main__':
    zope.testrunner.run([
        '--test-path', %r, '-c',
        ])
"""


class custom_test(test):
    # The zope.testrunner tests MUST be run using its own testrunner. This is
    # because its subprocess testing will call the script it was run with. We
    # therefore create a script to run the testrunner, and call that.
    def run(self):
        dist = self.distribution

        if dist.install_requires:
            dist.fetch_build_eggs(dist.install_requires)

        if dist.tests_require:
            dist.fetch_build_eggs(dist.tests_require)

        self.with_project_on_sys_path(self.run_tests)

    def run_tests(self):
        import tempfile
        script = CUSTOM_TEST_TEMPLATE % (
            sys.path, os.path.abspath(os.curdir), os.path.abspath('src'))
        _fd, filename = tempfile.mkstemp(prefix='temprunner', text=True)
        with open(filename, 'w') as scriptfile:
            scriptfile.write(script)

        import subprocess
        process = subprocess.Popen([sys.executable, filename])
        rc = process.wait()
        try:
            os.unlink(filename)
        except OSError as e:
            # This happens on Windows and I don't understand _why_.
            sys.stderr.write("Failed to clean up temporary script %s:\n%s: %s"
                             % (filename, e.__class__.__name__, e))
            sys.stderr.flush()
        sys.exit(rc)


def read(*names):
    with open(os.path.join(*names)) as f:
        return f.read()


long_description = (
    read('README.rst')
    + '\n\n' +
    read("docs", 'getting-started.rst')
    + '\n\n' +
    read('CHANGES.rst')
)

setup(
    name='zope.testrunner',
    version=version,
    url='https://github.com/zopefoundation/zope.testrunner',
    license='ZPL 2.1',
    description='Zope testrunner script.',
    long_description=long_description,
    author='Zope Foundation and Contributors',
    author_email='zope-dev@zope.org',
    packages=[
        "zope",
        "zope.testrunner",
    ],
    package_dir={'': 'src'},
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Framework :: Zope :: 3",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Zope Public License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Software Development :: Testing",
    ],
    namespace_packages=['zope'],
    python_requires='>=2.7,!=3.0.*,!=3.1.*,!=3.2.*,!=3.3.*,!=3.4.*',
    install_requires=INSTALL_REQUIRES,
    tests_require=TESTS_REQUIRE,
    extras_require=EXTRAS_REQUIRE,
    entry_points={
        'console_scripts':
            ['zope-testrunner = zope.testrunner:run'],
        'distutils.commands': [
            'ftest = zope.testrunner.eggsupport:ftest'],
    },
    include_package_data=True,
    zip_safe=False,
    cmdclass={
        'test': custom_test,
    },
)



##############################################################################
#
# Copyright (c) 2005 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

import unittest


class Test(unittest.TestCase):

    def test1(self):
        self.assertEqual(1, 1)

    def test2(self):
        self.assertEqual(1, 1)


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(Test))
    return suite
